
#include "tests.h"

using namespace boost;

// <summary>
// Enter vertices and edges into a graph,
// then extract both and print.
// </summary>
// <remarks>
// This is the first trial of the Boost Graph Library.<br/>
// This code failed on Visual Studio 7.<br/>
// => log the problem and send in to Boost.
// </remarks>
void Tests::Test1(void)
{
   // Types for this routine.
   typedef std::pair<int, int>                         E;           // Edge
   typedef adjacency_list<vecS, vecS, bidirectionalS>  Graph;       // Graph.
   typedef graph_traits<Graph>::vertex_iterator        VertexIter;  // Vertex iterator.
   typedef graph_traits<Graph>::edge_iterator          EdgeIter;    // Edge iterator.
   typedef property_map<Graph, vertex_index_t>::type   IndexMap;    // Map of vertex numbers from from iterator.

   
   std::cout << "Tests::Test1" << std::endl;

   // write out the edges in the graph
   const int num_edges = 11;
   E edge_array[num_edges] 
      = { E(0,1), E(0,2), E(0,3), E(0,4), 
          E(2,0), E(3,0), E(2,4), E(3,1), 
          E(3,4), E(4,0), E(4,1) };
   const int num_vertices = 5;

   // declare a graph object
   Graph g(num_vertices);

   // add the edges to the graph object
   for (int i = 0; i < num_edges; ++i)
   {
      add_edge(edge_array[i].first, edge_array[i].second, g);
   }

   // get the property map for vertex indices
   IndexMap index = get(vertex_index, g);
   
   // Print out the vertices.

   std::cout << "vertices(g) = " << std::endl;
   
   // Method-1 (said to be risky for VS)
   VertexIter vi, vi_end;
   for (tie(vi,vi_end) = vertices(g); vi != vi_end; ++vi)
   {
      std::cout << (int) index[*vi] <<  " ";
   }
   std::cout << std::endl;


   // Method-2
   std::pair<VertexIter, VertexIter> vp;
   for (vp = vertices(g); vp.first != vp.second; ++vp.first)
   {
      std::cout << (int) index[*vp.first] <<  " ";
   }
   std::cout << std::endl;
   std::cout << std::endl;

   // Print out the edges.

   std::cout << "edges(g) = " << std::endl;

   // Method-1 (risky for VS)
   EdgeIter ei, ei_end;
   for (tie(ei, ei_end) = edges(g); ei != ei_end; ++ei)
   {
      std::cout << "(" << (int) index[source(*ei, g)] 
                << "," << (int) index[target(*ei, g)] << ") ";
   }
   std::cout << std::endl;

   // Method-2
   std::pair<EdgeIter, EdgeIter> ep;
   for (ep = edges(g); ep.first != ep.second; ++ep.first)
   {
      std::cout << "(" << (int) index[source(*ep.first, g)] 
                << "," << (int) index[target(*ep.first, g)] << ") ";
   }
   std::cout << std::endl;
   std::cout << std::endl;



   std::cout << std::endl;
}

// <summary>
// </summary>
// <remarks>
// </remarks>
void Tests::Test2(void)
{
   std::cout << "Tests::Test2" << std::endl;
}
